from OFS import ZDOM
import string

def only_element_nodes(nodelist):
    """Only extract element nodes from a list of nodes.

    nodelist - the list of nodes
    result   - a list with only element nodes in it; all other nodes filtered
               out
    """
    ret = []
    for node in nodelist:
	nodetype = node.getNodeType()
        if nodetype == ZDOM.ELEMENT_NODE:
            ret.append(node)
        elif nodetype == ZDOM.COMMENT_NODE:
            continue
        elif (nodetype == ZDOM.TEXT_NODE and
              string.strip(node.getNodeValue()) == ""):
            continue
        else:
            raise "Illegal Document",\
                  "Unexpected non-element nodes found."
    return ret

def insertTop(node, name):
    """Insert a new element node at the top of the list of this one.
    node   - the node at the top of which to insert
    name   - the name of the new node to insert
    result - the new node
    """
    # create new node
    newNode = createElementNode(node, name)
    # insert it at the top of the node
    newNode = node.insertBefore(newNode, node.getFirstChild())
    # return the new node
    return newNode

def insertBottom(node, name):
    """Insert a new element node at the bottom of the list of this one.
    node   - the node at the bottom of which to insert
    name   - the name of the new node to insert
    result - the new node
    """
    # create new node
    newNode = createElementNode(node, name)
    # insert it at the bottom of the node
    newNode = node.appendChild(newNode)
    # return the new node
    return newNode

def insertAbove(node, name):
    """Insert a new element node above this one.
    node   - the node above which to insert
    name   - the name of the new node to insert
    result - the new node
    """
    # create new node
    newNode = createElementNode(node, name)
    # insert it above node
    newNode = node.getParentNode().insertBefore(newNode, node)
    # return the new node
    return newNode

def insertBelow(node, name):
    """Insert a new element node below this one.
    node   - the node below which to insert
    name   - the name of the new node to insert
    result - the new node
    """
    # create new node
    newNode = createElementNode(node, name)
    # insert it below node
    newNode = node.getParentNode().insertBefore(newNode, node.getNextSibling())
    # return the new node
    return newNode

def createElementNode(node, name):
    """Create a new element node.
    node   - node in the document in which to create
    name   - name of the new element node
    result - the newly created element node
    """
    return node.getOwnerDocument().createElement(name)

def findAbove(node, name=None):
    """Find an element node above this one.
    node   - the node above which to search
    name   - name of element node to look for, or any element node if None
    result - node that's found, or None if no node can be found above this one
    """
    foundNode = node
    while 1:
	# look upwards
	foundNode = foundNode.getPreviousSibling()
	# if we find no more nodes, foundNode is none, return None
	if foundNode is None:
	    return None
	# if this is the right node, return it
	if (foundNode.getNodeType() == ZDOM.ELEMENT_NODE and 
	    (name is None or foundNode.getNodeName() == name)):
	    return foundNode

def findBelow(node, name=None):
    """Find an element node below this one.
    node   - the node below which to search 
    name   - name of element node to look for, or any element node if None
    result - node that's found, or None if no node can be found below this one
    """
    foundNode = node
    while 1:
	# look downwards
	foundNode = foundNode.getNextSibling()
	# if we find no more nodes, foundNode is none, return None
	if foundNode is None:
	    return None
	# if this is the right node, return it
	if (foundNode.getNodeType() == ZDOM.ELEMENT_NODE and 
	    (name is None or foundNode.getNodeName() == name)):
	    return foundNode	
    
def setText(node, text):
    """Set text contents for element node, removes previous nodes inside.
    node - the node in which to set the text node
    text - the text in the text node
    """
    # remove previous child nodes
    for child in node.getChildNodes():
	node.removeChild(child)
    # add a new textnode with new text
    doc = node.getOwnerDocument()
    node.appendChild(doc.createTextNode(text))
