import string

class Error(Exception):
    pass

class XMLWidgetBase:
    """Base class for all XML Widgets.
    """
    def __init__(self):
	self._events = {}
    
    def allow_event(self, event_name):
	"""Allow a certain event to occur on this widget.

	event_name - the event that is allowed
	"""
	if not hasattr(self, event_name):
	    raise Error, "No method with event name '%s' exists"\
	                 "on this widget." % event_name
    
	# add method to dict
	self._events[event_name] = None

    def render(self, manager, node, REQUEST):
	"""Render node.
	
	manager - the XMLWidget manager that is used to render
	node    - the node to render with this widget
	REQUEST - the Zope REQUEST
	result  - rendered HTML string
	"""
	raise Error, "Unable to render XMLWidgetBase!"
    
    def handle_event(self, event_name, manager, node, REQUEST):
	"""Handle an event passed to this widget.
	
	event_name - the name of the event method to handle this event
	manager    - XMLWidgetManager used to render
	node       - the node this event occurs on
	REQUEST    - Zope REQUEST
	
	NOTE: events do not have any directly visible result; only the
	manager and the XMLDocument can be influenced. Rerender the 
	XMLDocument using the manager to see the result.
	"""

	# if event_name is not one of the allowed events
	if not self._events.has_key(event_name):
	    events = self._events.keys()
	    if events != []:
		allowed_events = string.join(events, ", ")
		raise Error, "Event '%s' is unknown on this widget. The "\
		      "following events are allowed: %s" % (event_name, 
							    allowed_events)
	    else:
		raise Error, "Event '%s' is unknown on this widget. No "\
		      "events at all exist for this widget." % (event_name)

	# otherwise, call event with node and REQUEST on this widget
	getattr(self, event_name)(manager, node, REQUEST)
	
