from XMLWidget import XMLWidgetBase
from cStringIO import StringIO
import XMLSupport
from OFS import ZDOM
from DateTime import DateTime
import string

class Error(Exception):
    """Simple exception class for this module."""
    pass

def configure(manager):
    """Configure manager to deal with structure
    """
    # get widgetfinder (that keeps map from XML node -> XML Widget)
    wf = manager.widgetfinder
    # clear widget finder from previous contents
    wf.clearWidgets()
    wf.clearSpecialWidgets()

    # define structure for rendering

    wf.setWidget(StructureWidget(),
                 (ZDOM.ELEMENT_NODE, 'structure'),
		 None)
    wf.setWidget(LevelWidget(),
                 (ZDOM.ELEMENT_NODE, 'level'),
		 None)
    wf.setWidget(DescWidget(),
                 (ZDOM.ELEMENT_NODE, 'desc'),
		 (ZDOM.ELEMENT_NODE, 'level'))

class StructureWidget(XMLWidgetBase):
    """Widget for 'structure' node.
    """
    def render(self, manager, node, REQUEST):
	result = StringIO()
	# render title
	result.write("<h3>%s</h3>\n" % node.title)
	# render children
	result.write(render_children(manager, node, REQUEST))
	# return string
	return result.getvalue()

class LevelWidget(XMLWidgetBase):
    """Widget for 'level' node.
    """ 
    def getdepth(self, node):
	counter = 0
	while 1:
	    node = node.getParentNode()
	    if node.getNodeName() == 'level':
		counter = counter + 1
	    else:
		break
	return counter

    def render(self, manager, node, REQUEST):
	result = StringIO()
      	# render name
	result.write("<p><strong>%s</strong></p>\n" % node.name)
	# render desc
	children = XMLSupport.only_element_nodes(node.getChildNodes())

	# child 0 is supposed to be desc
	result.write("<p>%s</p>" % children[0].text_content())
	
	# list links
	result.write("<hr>")
    
	for child in children[1:]:
	    result.write('<p><a href="%s/display">%s</a></p>' % 
			 (child.absolute_url(), child.name))
	
	return result.getvalue()
	# start sublist
	#result.write('''
	#<ul>
	#''')

	# render rest of children
	#for child in children[1:]:
	#    result.write(manager.render(child, REQUEST))

	# end sublist
	#result.write('''
	#</ul>
	#''')

	# return string
	return result.getvalue()

class DescWidget(XMLWidgetBase):
    """Widget for 'desc' node.
    """
    def render(self, manager, node, REQUEST):
	result = StringIO()
	# render text
	result.write("<p>%s<p>\n" % node.text_content())
	# return string
	return result.getvalue()

def render_children(manager, node, REQUEST):
    """Renders element child nodes.
    """
    result = StringIO()
    children = XMLSupport.only_element_nodes(node.getChildNodes())
    for child in children:
	result.write(manager.render(child, REQUEST))
    return result.getvalue()
